@@ -10,13 +10,13 @@ use MooX::StrictConstructor;
 use MooX::Types::MooseLike::Base qw(Str);
 use namespace::autoclean;
 
-our $VERSION = '1.018';
+our $VERSION = '1.019';
 
 with qw(
     Locale::TextDomain::OO::Role::Logger
 );
 
-my $is_too_late;
+my $loaded_plugins;
 sub load_plugins {
     my ( $class, @args ) = @_;
 
@@ -25,16 +25,28 @@ sub load_plugins {
     if ( $plugins ) {
         ref $plugins eq 'ARRAY'
             or confess 'Attribute plugins expected as ArrayRef';
-        for my $plugin ( @{$plugins} ) {
-            $is_too_late
-                and confess 'Too late to load plugins, another method new was called before';
-            my $package = ( 0 == index $plugin, q{+} )
-                ? $plugin
-                : "Locale::TextDomain::OO::Plugin::$plugin";
-            with $package;
+
+        my $current_plugins = join ', ', sort @{$plugins};
+        if ( defined $loaded_plugins ) {
+            length $current_plugins
+                and $loaded_plugins ne $current_plugins
+                and confess
+                    "Too late to load plugins $current_plugins.",
+                    " Another method new was called before with plugins $loaded_plugins";
+        }
+        else {
+            for my $plugin ( @{$plugins} ) {
+                my $package = ( 0 == index $plugin, q{+} )
+                    ? $plugin
+                    : "Locale::TextDomain::OO::Plugin::$plugin";
+                with $package;
+            }
+            $loaded_plugins = $current_plugins;
         }
     }
-    $is_too_late = 1;
+    if ( ! defined $loaded_plugins ) {
+        $loaded_plugins = q{};
+    }
 
     return \%arg_of;
 }
@@ -236,13 +248,13 @@ __END__
 
 Locale::TextDomain::OO::Translator - Translator class
 
-$Id: Translator.pm 573 2015-02-07 20:59:51Z steffenw $
+$Id: Translator.pm 580 2015-04-12 20:09:18Z steffenw $
 
 $HeadURL: svn+ssh://steffenw@svn.code.sf.net/p/perl-gettext-oo/code/module/trunk/lib/Locale/TextDomain/OO/Translator.pm $
 
 =head1 VERSION
 
-1.018
+1.019
 
 =head1 DESCRIPTION
 
